@@ -7,15 +7,24 @@ from .element import Element, transform
 from .parser import parse
 
 
+class ModelBase:
+    @classmethod
+    def parse(cls, raw: dict):
+        raise NotImplementedError
+
+    def dump(self) -> dict:
+        raise NotImplementedError
+
+
 class ChannelType(IntEnum):
     TEXT = 0
-    VOICE = 1
+    DIRECT = 1
     CATEGORY = 2
-    DIRECT = 3
+    VOICE = 3
 
 
 @dataclass
-class Channel:
+class Channel(ModelBase):
     id: str
     type: ChannelType
     name: Optional[str] = None
@@ -37,7 +46,7 @@ class Channel:
 
 
 @dataclass
-class Guild:
+class Guild(ModelBase):
     id: str
     name: Optional[str] = None
     avatar: Optional[str] = None
@@ -56,9 +65,10 @@ class Guild:
 
 
 @dataclass
-class User:
+class User(ModelBase):
     id: str
     name: Optional[str] = None
+    nick: Optional[str] = None
     avatar: Optional[str] = None
     is_bot: Optional[bool] = None
 
@@ -70,6 +80,8 @@ class User:
         res: Dict[str, Any] = {"id": self.id}
         if self.name:
             res["name"] = self.name
+        if self.nick:
+            res["nick"] = self.nick
         if self.avatar:
             res["avatar"] = self.avatar
         if self.is_bot:
@@ -78,8 +90,9 @@ class User:
 
 
 @dataclass
-class Member:
+class Member(ModelBase):
     user: Optional[User] = None
+    nick: Optional[str] = None
     name: Optional[str] = None
     avatar: Optional[str] = None
     joined_at: Optional[datetime] = None
@@ -88,7 +101,7 @@ class Member:
     def parse(cls, raw: dict):
         data = raw.copy()
         if "user" in raw:
-            data["user"] = User(**raw["user"])
+            data["user"] = User.parse(raw["user"])
         if "joined_at" in raw:
             data["joined_at"] = datetime.fromtimestamp(int(raw["joined_at"]) / 1000)
         return cls(**data)
@@ -97,8 +110,8 @@ class Member:
         res = {}
         if self.user:
             res["user"] = self.user.dump()
-        if self.name:
-            res["name"] = self.name
+        if self.nick or self.name:
+            res["nick"] = self.nick or self.name
         if self.avatar:
             res["avatar"] = self.avatar
         if self.joined_at:
@@ -107,7 +120,7 @@ class Member:
 
 
 @dataclass
-class Role:
+class Role(ModelBase):
     id: str
     name: Optional[str] = None
 
@@ -131,7 +144,7 @@ class LoginStatus(IntEnum):
 
 
 @dataclass
-class Login:
+class Login(ModelBase):
     status: LoginStatus
     user: Optional[User] = None
     self_id: Optional[str] = None
@@ -157,19 +170,27 @@ class Login:
 
 
 @dataclass
-class ArgvInteraction:
+class ArgvInteraction(ModelBase):
     name: str
     arguments: list
     options: Any
 
+    @classmethod
+    def parse(cls, raw: dict):
+        return cls(**raw)
+
     def dump(self):
         return asdict(self)
 
 
 @dataclass
-class ButtonInteraction:
+class ButtonInteraction(ModelBase):
     id: str
 
+    @classmethod
+    def parse(cls, raw: dict):
+        return cls(**raw)
+
     def dump(self):
         return asdict(self)
 
@@ -183,20 +204,20 @@ class Opcode(IntEnum):
 
 
 @dataclass
-class Identify:
+class Identify(ModelBase):
     token: Optional[str] = None
     sequence: Optional[int] = None
 
 
 @dataclass
-class Ready:
+class Ready(ModelBase):
     logins: List[Login]
 
 
 @dataclass
-class Message:
+class MessageObject(ModelBase):
     id: str
-    content: List[Element]
+    content: str
     channel: Optional[Channel] = None
     guild: Optional[Guild] = None
     member: Optional[Member] = None
@@ -204,20 +225,38 @@ class Message:
     created_at: Optional[datetime] = None
     updated_at: Optional[datetime] = None
 
+    @classmethod
+    def from_elements(
+        cls,
+        id: str,
+        content: List[Element],
+        channel: Optional[Channel] = None,
+        guild: Optional[Guild] = None,
+        member: Optional[Member] = None,
+        user: Optional[User] = None,
+        created_at: Optional[datetime] = None,
+        updated_at: Optional[datetime] = None,
+    ):
+        return cls(id, "".join(str(i) for i in content), channel, guild, member, user, created_at, updated_at)
+
+    @property
+    def message(self) -> List[Element]:
+        return transform(parse(self.content))
+
     @classmethod
     def parse(cls, raw: dict):
         data = {
             "id": raw["id"],
-            "content": transform(parse(raw["content"])),
+            "content": raw["content"],
         }
         if "channel" in raw:
-            data["channel"] = Channel(**raw["channel"])
+            data["channel"] = Channel.parse(raw["channel"])
         if "guild" in raw:
-            data["guild"] = Guild(**raw["guild"])
+            data["guild"] = Guild.parse(raw["guild"])
         if "member" in raw:
             data["member"] = Member.parse(raw["member"])
         if "user" in raw:
-            data["user"] = User(**raw["user"])
+            data["user"] = User.parse(raw["user"])
         if "created_at" in raw:
             data["created_at"] = datetime.fromtimestamp(int(raw["created_at"]) / 1000)
         if "updated_at" in raw:
@@ -225,7 +264,7 @@ class Message:
         return cls(**data)
 
     def dump(self):
-        res: Dict[str, Any] = {"id": self.id, "content": "".join(map(str, self.content))}
+        res: Dict[str, Any] = {"id": self.id, "content": self.content}
         if self.channel:
             res["channel"] = self.channel.dump()
         if self.guild:
@@ -254,11 +293,14 @@ class Event:
     guild: Optional[Guild] = None
     login: Optional[Login] = None
     member: Optional[Member] = None
-    message: Optional[Message] = None
+    message: Optional[MessageObject] = None
     operator: Optional[User] = None
     role: Optional[Role] = None
     user: Optional[User] = None
 
+    _type: Optional[str] = None
+    _data: Optional[dict] = None
+
     @classmethod
     def parse(cls, raw: dict):
         data = {
@@ -269,25 +311,29 @@ class Event:
             "timestamp": datetime.fromtimestamp(int(raw["timestamp"]) / 1000),
         }
         if "argv" in raw:
-            data["argv"] = ArgvInteraction(**raw["argv"])
+            data["argv"] = ArgvInteraction.parse(raw["argv"])
         if "button" in raw:
-            data["button"] = ButtonInteraction(**raw["button"])
+            data["button"] = ButtonInteraction.parse(raw["button"])
         if "channel" in raw:
-            data["channel"] = Channel(**raw["channel"])
+            data["channel"] = Channel.parse(raw["channel"])
         if "guild" in raw:
-            data["guild"] = Guild(**raw["guild"])
+            data["guild"] = Guild.parse(raw["guild"])
         if "login" in raw:
             data["login"] = Login.parse(raw["login"])
         if "member" in raw:
             data["member"] = Member.parse(raw["member"])
         if "message" in raw:
-            data["message"] = Message.parse(raw["message"])
+            data["message"] = MessageObject.parse(raw["message"])
         if "operator" in raw:
-            data["operator"] = User(**raw["operator"])
+            data["operator"] = User.parse(raw["operator"])
         if "role" in raw:
-            data["role"] = Role(**raw["role"])
+            data["role"] = Role.parse(raw["role"])
         if "user" in raw:
-            data["user"] = User(**raw["user"])
+            data["user"] = User.parse(raw["user"])
+        if "_type" in raw:
+            data["_type"] = raw["_type"]
+        if "_data" in raw:
+            data["_data"] = raw["_data"]
         return cls(**data)
 
     def dump(self):
@@ -318,14 +364,18 @@ class Event:
             res["role"] = self.role.dump()
         if self.user:
             res["user"] = self.user.dump()
+        if self._type:
+            res["_type"] = self._type
+        if self._data:
+            res["_data"] = self._data
         return res
 
 
-T = TypeVar("T")
+T = TypeVar("T", bound=ModelBase)
 
 
 @dataclass
-class PageResult(Generic[T]):
+class PageResult(ModelBase, Generic[T]):
     data: List[T]
     next: Optional[str] = None
 
@@ -333,3 +383,9 @@ class PageResult(Generic[T]):
     def parse(cls, raw: dict, parser: Callable[[dict], T]) -> "PageResult[T]":
         data = [parser(item) for item in raw["data"]]
         return cls(data, raw.get("next"))
+
+    def dump(self):
+        res: dict = {"data": [item.dump() for item in self.data]}
+        if self.next:
+            res["next"] = self.next
+        return res
